use super::*;

pub struct InliningMode;

impl Mode for InliningMode {
    type ParseResult<A, E, I> = Result<(A, I), E>;

    fn map_err<A, E0, E1, I>(
        result: Self::ParseResult<A, E0, I>,
        f: impl FnOnce(E0) -> E1,
    ) -> Self::ParseResult<A, E1, I> {
        result.map_err(f)
    }

    fn bind<A0, A1, E, I>(
        result: Self::ParseResult<A0, E, I>,
        f: impl FnOnce(A0) -> Result<A1, E>,
    ) -> Self::ParseResult<A1, E, I> {
        let (a0, i) = result?;
        let a1 = f(a0)?;
        Ok((a1, i))
    }
}

impl<'a, Ctx: Context<'a>, F: InlineableFactory<'a, Ctx>> FactoryProxy<'a, Ctx>
    for WithMode<F, InliningMode>
{
    type F = F;

    fn pdeserialize(f: &Self::F, inctx: impl InCtx<'a, Ctx>) -> ParseResult<'a, Ctx, Self::F> {
        f.ideserialize(inctx).seal()
    }

    fn pextend(
        f: &Self::F,
        _mentionable: Mtbl<'a, Ctx, Self::F>,
        tail: &[u8],
    ) -> ParseResult<'a, Ctx, Self::F> {
        Err(f.extension_error(tail))
    }
}

impl<A: InlineableAtomic> AtomicProxy for WithMode<A, InliningMode> {
    type A = A;

    fn pa_deserialize(inlining: impl Inlining) -> AParseResult<Self::A> {
        A::a_ideserialize(inlining).seal()
    }

    fn pa_extend(_a: Self::A, tail: &[u8]) -> AParseResult<Self::A> {
        Err(A::a_extension_error(tail))
    }
}
